=== modified file 'NEWS'
--- NEWS	2005-09-05 09:11:03 +0000
+++ NEWS	2005-09-12 01:29:49 +0000
@@ -1,10 +1,32 @@
 development head
 
+  IMPROVEMENTS:
+
+    * Adding a file whose parent directory is not versioned will
+      implicitly add the parent, and so on up to the root. This means
+      you should never need to explictly add a directory, they'll just
+      get added when you add a file in the directory.  Contributed by
+      Michael Ellerman.
+
+
   INTERNALS:
 
     * Refactor xml packing/unpacking.
 
 
+  BUG FIXES: 
+
+    * Fixed 'bzr mv' by Ollie Rutherfurd.
+
+    * Fixed strange error when trying to access a nonexistent http
+      branch.
+
+
+  PORTABILITY:
+
+    * Various Windows fixes from Ollie Rutherfurd.
+
+
 bzr-0.0.7 2005-09-02
 
   NEW FEATURES:

=== modified file 'bzr'
--- bzr	2005-08-25 07:46:11 +0000
+++ bzr	2005-09-09 14:17:50 +0000
@@ -40,7 +40,8 @@
     print >>sys.stderr, "bzr: error: cannot find a suitable python interpreter"
     print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
     sys.exit(1)
-os.unsetenv(REINVOKE)
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
 
 import bzrlib
 import bzrlib.commands

=== modified file 'bzr-man.py'
--- bzr-man.py	2005-06-06 13:24:18 +0000
+++ bzr-man.py	2005-09-09 14:17:50 +0000
@@ -42,7 +42,8 @@
     print >>sys.stderr, "bzr-man.py: error: cannot find a suitable python interpreter"
     print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
     sys.exit(1)
-os.unsetenv(REINVOKE)
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
 
 import bzrlib, bzrlib.help
 

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-09-09 11:37:44 +0000
+++ bzrlib/add.py	2005-09-14 05:22:13 +0000
@@ -18,6 +18,7 @@
 from bzrlib.errors import NotBranchError
 from bzrlib.branch import Branch
 from bzrlib.osutils import quotefn
+from os.path import dirname
 
 def glob_expand_for_win32(file_list):
     import glob
@@ -78,11 +79,8 @@
     Returns the number of files added.
     """
     import os
-    import sys
-    from bzrlib.osutils import quotefn
     from bzrlib.errors import BadFileKindError, ForbiddenFileError
     import bzrlib.branch
-    import bzrlib.osutils
 
     assert isinstance(recurse, bool)
 
@@ -129,10 +127,7 @@
         elif sub_tree:
             mutter("%r is a bzr tree" %f)
         else:
-            entry = inv.add_path(rf, kind=kind)
-            mutter("added %r kind %r file_id={%s}" % (rf, kind, entry.file_id))
-            count += 1 
-            reporter(rf, kind, entry)
+            count += __add_one(branch, inv, rf, kind, reporter)
 
         if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
@@ -152,3 +147,21 @@
         branch._write_inventory(inv)
 
     return count
+
+def __add_one(branch, inv, path, kind, reporter):
+    """Add a file or directory, automatically add unversioned parents."""
+
+    # Nothing to do if path is already versioned.
+    # This is safe from infinite recursion because the branch root is
+    # always versioned.
+    if inv.path2id(path) != None:
+        return 0
+
+    # add parent
+    count = __add_one(branch, inv, dirname(path), 'directory', reporter)
+
+    entry = inv.add_path(path, kind=kind)
+    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
+    reporter(path, kind, entry)
+
+    return count + 1

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-09 18:59:17 +0000
+++ bzrlib/branch.py	2005-09-14 05:22:13 +0000
@@ -185,7 +185,7 @@
     def __init__(self, base, init=False, find_root=True):
         """Create new branch object at a particular location.
 
-        base -- Base directory for the branch.
+        base -- Base directory for the branch. May be a file:// url.
         
         init -- If True, create new control files in a previously
              unversioned directory.  If False, the branch must already
@@ -204,6 +204,8 @@
         elif find_root:
             self.base = find_branch_root(base)
         else:
+            if base.startswith("file://"):
+                base = base[7:]
             self.base = os.path.realpath(base)
             if not isdir(self.controlfilename('.')):
                 raise NotBranchError("not a bzr branch: %s" % quotefn(base),
@@ -229,7 +231,6 @@
             warn("branch %r was not explicitly unlocked" % self)
             self._lock.unlock()
 
-
     def lock_write(self):
         if self._lock_mode:
             if self._lock_mode != 'w':
@@ -816,11 +817,22 @@
         """Pull in all new revisions from other branch.
         """
         from bzrlib.fetch import greedy_fetch
+        from bzrlib.revision import get_intervening_revisions
 
         pb = bzrlib.ui.ui_factory.progress_bar()
         pb.update('comparing histories')
 
-        revision_ids = self.missing_revisions(other, stop_revision)
+        try:
+            revision_ids = self.missing_revisions(other, stop_revision)
+        except DivergedBranches, e:
+            try:
+                if stop_revision is None:
+                    end_revision = other.last_patch()
+                revision_ids = get_intervening_revisions(self.last_patch(), 
+                                                         end_revision, other)
+                assert self.last_patch() not in revision_ids
+            except bzrlib.errors.NotAncestor:
+                raise e
 
         if len(revision_ids) > 0:
             count = greedy_fetch(self, other, revision_ids[-1], pb)[0]
@@ -1191,6 +1203,7 @@
             raise InvalidRevisionNumber(revno)
         
         
+        
 
 
 class ScratchBranch(LocalBranch):
@@ -1343,6 +1356,4 @@
     merge((to_location, -1), (to_location, 0), this_dir=to_location,
           check_clean=False, ignore_zero=True)
     
-    from_location = branch_from.base
     br_to.set_parent(branch_from.base)
-

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-13 09:52:12 +0000
+++ bzrlib/builtins.py	2005-09-14 05:22:13 +0000
@@ -62,11 +62,10 @@
     directory is shown.  Otherwise, only the status of the specified
     files or directories is reported.  If a directory is given, status
     is reported for everything inside that directory.
+    """
 
-    If a revision is specified, the changes since that revision are shown.
-    """
     takes_args = ['file*']
-    takes_options = ['all', 'show-ids', 'revision']
+    takes_options = ['all', 'show-ids']
     aliases = ['st', 'stat']
     
     def run(self, all=False, show_ids=False, file_list=None):
@@ -145,8 +144,10 @@
     Therefore simply saying 'bzr add' will version all files that
     are currently unknown.
 
-    TODO: Perhaps adding a file whose directly is not versioned should
-    recursively add that parent, rather than giving an error?
+    Adding a file whose parent directory is not versioned will
+    implicitly add the parent, and so on up to the root. This means
+    you should never need to explictly add a directory, they'll just
+    get added when you add a file in the directory.
     """
     takes_args = ['file*']
     takes_options = ['verbose', 'no-recurse']
@@ -274,8 +275,8 @@
             if len(names_list) != 2:
                 raise BzrCommandError('to mv multiple files the destination '
                                       'must be a versioned directory')
-            for pair in b.move(rel_names[0], rel_names[1]):
-                print "%s => %s" % pair
+            b.rename_one(rel_names[0], rel_names[1])
+            print "%s => %s" % (rel_names[0], rel_names[1])
             
     
 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-09-13 09:52:12 +0000
+++ bzrlib/commands.py	2005-09-14 05:22:13 +0000
@@ -25,6 +25,13 @@
 # would help with validation and shell completion.
 
 
+# TODO: Help messages for options.
+
+# TODO: Define arguments by objects, rather than just using names.
+# Those objects can specify the expected type of the argument, which
+# would help with validation and shell completion.
+
+
 
 import sys
 import os
@@ -354,8 +361,6 @@
     return parsed
 
 
-
-
 # list of all available options; the rhs can be either None for an
 # option that takes no argument, or a constructor function that checks
 # the type.

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-09-13 09:52:12 +0000
+++ bzrlib/diff.py	2005-09-14 05:22:13 +0000
@@ -144,7 +144,7 @@
 
 
 def show_diff(b, revision, specific_files, external_diff_options=None,
-              revision2=None):
+              revision2=None, output=None):
     """Shortcut for showing the diff to the working tree.
 
     b
@@ -156,7 +156,9 @@
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
     """
-    import sys
+    if output is None:
+        import sys
+        output = sys.stdout
 
     if revision is None:
         old_tree = b.basis_tree()
@@ -168,7 +170,7 @@
     else:
         new_tree = b.revision_tree(revision2.in_branch(b).rev_id)
 
-    show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
+    show_diff_trees(old_tree, new_tree, output, specific_files,
                     external_diff_options)
 
 

=== added file 'bzrlib/graph.py'
--- bzrlib/graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/graph.py	2005-09-12 02:53:07 +0000
@@ -0,0 +1,93 @@
+# (C) 2005 Canonical
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+def max_distance(node, ancestors, distances, root_descendants):
+    """Calculate the max distance to an ancestor.  
+    Return None if not all possible ancestors have known distances"""
+    best = None
+    if node in distances:
+        best = distances[node]
+    for ancestor in ancestors[node]:
+        # skip ancestors we will never traverse:
+        if root_descendants is not None and ancestor not in root_descendants:
+            continue
+        # An ancestor which is not listed in ancestors will never be in
+        # distances, so we pretend it never existed.
+        if ancestor not in ancestors:
+            continue
+        if ancestor not in distances:
+            return None
+        if best is None or distances[ancestor] > best:
+            best = distances[ancestor] + 1
+    return best
+
+    
+def node_distances(graph, ancestors, start, root_descendants=None):
+    """Produce a list of nodes, sorted by distance from a start node.
+    This is an algorithm devised by Aaron Bentley, because applying Dijkstra
+    backwards seemed too complicated.
+
+    For each node, we walk its descendants.  If all the descendant's ancestors
+    have a max-distance-to-start, (excluding ones that can never reach start),
+    we calculate their max-distance-to-start, and schedule their descendants.
+
+    So when a node's last parent acquires a distance, it will acquire a
+    distance on the next iteration.
+
+    Once we know the max distances for all nodes, we can return a list sorted
+    by distance, farthest first.
+    """
+    distances = {start: 0}
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            line_descendants = graph[line]
+            assert line not in line_descendants, "%s refers to itself" % line
+            for descendant in line_descendants:
+                distance = max_distance(descendant, ancestors, distances,
+                                        root_descendants)
+                if distance is None:
+                    continue
+                distances[descendant] = distance
+                new_lines.add(descendant)
+        lines = new_lines
+    return distances
+
+def farthest_nodes(graph, ancestors, start):
+    def by_distance(n):
+        return distances[n],n
+
+    distances = node_distances(graph, ancestors, start)
+    node_list = distances.keys()
+    node_list.sort(key=by_distance, reverse=True)
+    return node_list
+
+def all_descendants(descendants, start):
+    result = set()
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            if line not in descendants:
+                continue
+            for descendant in descendants[line]:
+                if descendant in result:
+                    continue
+                result.add(descendant)
+                new_lines.add(descendant)
+        lines = new_lines
+    return result

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-09-05 04:08:03 +0000
+++ bzrlib/help.py	2005-09-06 11:09:03 +0000
@@ -41,7 +41,7 @@
   bzr diff      show detailed diffs
 
   bzr merge     pull in changes from another branch
-  bzr commit    
+  bzr commit    save some or all changes
 
   bzr log       show history of changes
   bzr check     validate storage

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-09-01 11:19:08 +0000
+++ bzrlib/osutils.py	2005-09-09 14:17:50 +0000
@@ -407,21 +407,27 @@
     """Return size of given open file."""
     return os.fstat(f.fileno())[ST_SIZE]
 
-
-if hasattr(os, 'urandom'): # python 2.4 and later
+# Define rand_bytes based on platform.
+try:
+    # Python 2.4 and later have os.urandom,
+    # but it doesn't work on some arches
+    os.urandom(1)
     rand_bytes = os.urandom
-elif sys.platform == 'linux2':
-    rand_bytes = file('/dev/urandom', 'rb').read
-else:
-    # not well seeded, but better than nothing
-    def rand_bytes(n):
-        import random
-        s = ''
-        while n:
-            s += chr(random.randint(0, 255))
-            n -= 1
-        return s
-
+except (NotImplementedError, AttributeError):
+    # If python doesn't have os.urandom, or it doesn't work,
+    # then try to first pull random data from /dev/urandom
+    if os.path.exists("/dev/urandom"):
+        rand_bytes = file('/dev/urandom', 'rb').read
+    # Otherwise, use this hack as a last resort
+    else:
+        # not well seeded, but better than nothing
+        def rand_bytes(n):
+            import random
+            s = ''
+            while n:
+                s += chr(random.randint(0, 255))
+                n -= 1
+            return s
 
 ## TODO: We could later have path objects that remember their list
 ## decomposition (might be too tricksy though.)

=== modified file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-09-09 15:03:42 +0000
+++ bzrlib/remotebranch.py	2005-09-14 05:22:13 +0000
@@ -28,6 +28,7 @@
 from cStringIO import StringIO
 import os
 import urllib2
+import urlparse
 
 from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
@@ -83,8 +84,8 @@
     orig_url = url
     while True:
         try:
-            ff = get_url(url + '/.bzr/branch-format')
-
+            fmt_url = url + '/.bzr/branch-format'
+            ff = get_url(fmt_url)
             fmt = ff.read()
             ff.close()
 
@@ -97,12 +98,18 @@
         except urllib2.URLError:
             pass
 
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
+        scheme, host, path = list(urlparse.urlparse(url))[:3]
+        # discard params, query, fragment
+        
+        # strip off one component of the path component
+        idx = path.rfind('/')
+        if idx == -1 or path == '/':
+            raise BzrError('no branch root found for URL %s'
+                           ' or enclosing directories'
+                           % orig_url)
+        path = path[:idx]
+        url = urlparse.urlunparse((scheme, host, path, '', '', ''))
+        
 
 
 class RemoteBranch(Branch):

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-09-06 07:26:13 +0000
+++ bzrlib/revision.py	2005-09-12 12:39:58 +0000
@@ -16,7 +16,7 @@
 
 
 import bzrlib.errors
-
+from bzrlib.graph import farthest_nodes, node_distances, all_descendants
 
 class RevisionReference(object):
     """
@@ -159,7 +159,7 @@
     return matches
 
 
-def common_ancestor(revision_a, revision_b, revision_source):
+def old_common_ancestor(revision_a, revision_b, revision_source):
     """Find the ancestor common to both revisions that is closest to both.
     """
     from bzrlib.trace import mutter
@@ -190,6 +190,73 @@
         raise bzrlib.errors.AmbiguousBase((a_closest[0], b_closest[0]))
     return a_closest[0]
 
+def revision_graph(revision, revision_source):
+    """Produce a graph of the ancestry of the specified revision.
+    Return root, ancestors map, descendants map
+
+    TODO: Produce graphs with the NULL revision as root, so that we can find
+    a common even when trees are not branches don't represent a single line
+    of descent.
+    """
+    ancestors = {}
+    descendants = {}
+    lines = [revision]
+    root = None
+    descendants[revision] = {}
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            try:
+                rev = revision_source.get_revision(line)
+                parents = [p.revision_id for p in rev.parents]
+                if len(parents) == 0:
+                    root = line
+            except bzrlib.errors.NoSuchRevision:
+                if line == revision:
+                    raise
+                parents = None
+            if parents is not None:
+                for parent in parents:
+                    if parent not in ancestors:
+                        new_lines.add(parent)
+                    if parent not in descendants:
+                        descendants[parent] = {}
+                    descendants[parent][line] = 1
+            if parents is not None:
+                ancestors[line] = set(parents)
+        lines = new_lines
+    assert root not in descendants[root]
+    assert root not in ancestors[root]
+    return root, ancestors, descendants
+
+def combined_graph(revision_a, revision_b, revision_source):
+    """Produce a combined ancestry graph.
+    Return graph root, ancestors map, descendants map, set of common nodes"""
+    root, ancestors, descendants = revision_graph(revision_a, revision_source)
+    root_b, ancestors_b, descendants_b = revision_graph(revision_b, 
+                                                        revision_source)
+    assert root == root_b
+    common = set()
+    for node, node_anc in ancestors_b.iteritems():
+        if node in ancestors:
+            common.add(node)
+        else:
+            ancestors[node] = set()
+        ancestors[node].update(node_anc)
+    for node, node_dec in descendants_b.iteritems():
+        if node not in descendants:
+            descendants[node] = set()
+        descendants[node].update(node_dec)
+    return root, ancestors, descendants, common
+
+def common_ancestor(revision_a, revision_b, revision_source):
+    root, ancestors, descendants, common = \
+        combined_graph(revision_a, revision_b, revision_source)
+    nodes = farthest_nodes(descendants, ancestors, root)
+    for node in nodes:
+        if node in common:
+            return node
+
 class MultipleRevisionSources(object):
     """Proxy that looks in multiple branches for revisions."""
     def __init__(self, *args):
@@ -214,49 +281,36 @@
     Otherwise, rev_id will be the last entry.  ancestor_id will never appear.
     If ancestor_id is not an ancestor, NotAncestor will be thrown
     """
-    [rev_source.get_revision(r) for r in (ancestor_id, rev_id)]
-    if ancestor_id == rev_id:
-        return []
-    def historical_lines(line):
-        """Return a tuple of historical/non_historical lines, for sorting.
-        The non_historical count is negative, since non_historical lines are
-        a bad thing.
-        """
-        good_count = 0
-        bad_count = 0
-        for revision in line:
-            if revision in revision_history:
-                good_count += 1
-            else:
-                bad_count -= 1
-        return good_count, bad_count
-    active = [[rev_id]]
-    successful_lines = []
-    while len(active) > 0:
-        new_active = []
-        for line in active:
-            parent_ids = [p.revision_id for p in 
-                          rev_source.get_revision(line[-1]).parents]
-            for parent in parent_ids:
-                line_copy = line[:]
-                if parent == ancestor_id:
-                    successful_lines.append(line_copy)
-                else:
-                    line_copy.append(parent)
-                    new_active.append(line_copy)
-        active = new_active
-    if len(successful_lines) == 0:
-        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
-    for line in successful_lines:
-        line.reverse()
-    if revision_history is not None:
-        by_historical_lines = []
-        for line in successful_lines:
-            count = historical_lines(line)
-            by_historical_lines.append((count, line))
-        by_historical_lines.sort()
-        if by_historical_lines[-1][0][0] > 0:
-            return by_historical_lines[-1][1]
-    assert len(successful_lines)
-    successful_lines.sort(cmp, len)
-    return successful_lines[-1]
+    root, ancestors, descendants = revision_graph(rev_id, rev_source)
+    if len(descendants) == 0:
+        raise NoSuchRevision(rev_source, rev_id)
+    if ancestor_id not in descendants:
+        rev_source.get_revision(ancestor_id)
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    root_descendants = all_descendants(descendants, ancestor_id)
+    root_descendants.add(ancestor_id)
+    if rev_id not in root_descendants:
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    distances = node_distances(descendants, ancestors, ancestor_id,
+                               root_descendants=root_descendants)
+
+    def best_ancestor(rev_id):
+        best = None
+        for anc_id in ancestors[rev_id]:
+            try:
+                distance = distances[anc_id]
+            except KeyError:
+                continue
+            if revision_history is not None and anc_id in revision_history:
+                return anc_id
+            elif best is None or distance > best[1]:
+                best = (anc_id, distance)
+        return best[0]
+
+    next = rev_id
+    path = []
+    while next != ancestor_id:
+        path.append(next)
+        next = best_ancestor(next)
+    path.reverse()
+    return path

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-05 09:11:03 +0000
+++ bzrlib/selftest/__init__.py	2005-09-12 08:21:24 +0000
@@ -74,6 +74,9 @@
         
         self._log_file_name = name
 
+    def run(self, result):
+        self.apply_redirected(None, None, None,
+                              unittest.TestCase.run, self, result)
         
     def tearDown(self):
         logging.getLogger('').removeHandler(self._log_hdlr)
@@ -138,9 +141,15 @@
         if stdin is None:
             stdin = StringIO("")
         if stdout is None:
-            stdout = self._log_file
+            if hasattr(self, "_log_file"):
+                stdout = self._log_file
+            else:
+                stdout = StringIO()
         if stderr is None:
-            stderr = self._log_file
+            if hasattr(self, "_log_file"):
+                stderr = self._log_file
+            else:
+                stderr = StringIO()
         real_stdin = sys.stdin
         real_stdout = sys.stdout
         real_stderr = sys.stderr
@@ -324,6 +333,7 @@
                    'bzrlib.selftest.test_xml',
                    'bzrlib.fetch',
                    'bzrlib.selftest.teststore',
+                   'bzrlib.selftest.testgraph',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-09 11:37:44 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-14 05:22:13 +0000
@@ -27,6 +27,7 @@
 """
 
 import sys
+import os
 
 from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
 from bzrlib.branch import Branch
@@ -122,7 +123,6 @@
         assert file('.bzrignore', 'rb').read() == '*.blah\ngarh\n'
 
     def test_revert(self):
-        import os
         self.runbzr('init')
 
         file('hello', 'wt').write('foo')
@@ -147,17 +147,19 @@
         os.rmdir('revertdir')
         self.runbzr('revert')
 
-    def skipped_test_mv_modes(self):
+    def test_mv_modes(self):
         """Test two modes of operation for mv"""
         from bzrlib.branch import Branch
         b = Branch.initialize('.')
         self.build_tree(['a', 'c', 'subdir/'])
+        self.run_bzr('add', self.test_dir)
         self.run_bzr('mv', 'a', 'b')
         self.run_bzr('mv', 'b', 'subdir')
         self.run_bzr('mv', 'subdir/b', 'a')
-        self.run_bzr('mv', 'a', 'b', 'subdir')
+        self.run_bzr('mv', 'a', 'c', 'subdir')
         self.run_bzr('mv', 'subdir/a', 'subdir/newa')
 
+
     def test_main_version(self):
         """Check output from version command and master option is reasonable"""
         # output is intentionally passed through to stdout so that we
@@ -194,11 +196,9 @@
 
     def test_merge(self):
         from bzrlib.branch import Branch
-        import os
         
         os.mkdir('a')
         os.chdir('a')
-
         self.example_branch()
         os.chdir('..')
         self.runbzr('branch a b')
@@ -217,11 +217,37 @@
         a = Branch.open('.')
         b = Branch.open('../b')
         a.get_revision_xml(b.last_patch())
-
         self.log('pending merges: %s', a.pending_merges())
         #        assert a.pending_merges() == [b.last_patch()], "Assertion %s %s" \
         #        % (a.pending_merges(), b.last_patch())
 
+    def test_pull(self):
+        """Pull changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        self.runbzr('commit -m blah --unchanged')
+        os.chdir('../a')
+        a = Branch('.')
+        b = Branch('../b')
+        assert a.revision_history() == b.revision_history()[:-1]
+        self.runbzr('pull ../b')
+        assert a.revision_history() == b.revision_history()
+        self.runbzr('commit -m blah2 --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m blah3 --unchanged')
+        self.runbzr('pull ../a', retcode=1)
+        os.chdir('../a')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah4 --unchanged')
+        os.chdir('../b')
+        self.runbzr('pull ../a')
+        assert a.revision_history()[-1] == b.revision_history()[-1]
+        
 
     def test_add_reports(self):
         """add command prints the names of added files."""
@@ -250,7 +276,6 @@
     def test_bzr(self):
         from os import chdir, mkdir
         from os.path import exists
-        import os
 
         runbzr = self.runbzr
         backtick = self.backtick

=== added file 'bzrlib/selftest/testgraph.py'
--- bzrlib/selftest/testgraph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testgraph.py	2005-09-10 23:46:35 +0000
@@ -0,0 +1,36 @@
+from bzrlib.selftest import TestCase
+from bzrlib.graph import farthest_nodes
+
+class TestBase(TestCase):
+    def edge_add(self, *args):
+        for start, end in zip(args[:-1], args[1:]):
+            if start not in self.graph:
+                self.graph[start] = {}
+            if end not in self.graph:
+                self.graph[end] = {}
+            self.graph[start][end] = 1
+
+    def setUp(self):
+        TestCase.setUp(self)
+        self.graph = {}
+        self.edge_add('A', 'B', 'C', 'D')
+        self.edge_add('A', 'E', 'F', 'C')
+        self.edge_add('A', 'G', 'H', 'I', 'B')
+        self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
+        self.edge_add('O', 'N')
+    
+    def test_farthest(self):
+        descendants = {'A':set()}
+        for node in self.graph:
+            for ancestor in self.graph[node]:
+                if ancestor not in descendants:
+                    descendants[ancestor] = set()
+                descendants[ancestor].add(node)
+        nodes = farthest_nodes(self.graph, descendants, 'A')
+        self.assertEqual(nodes[0], 'D')
+        assert nodes[1] in ('N', 'C')
+        assert nodes[2] in ('N', 'C')
+        assert nodes[3] in ('B', 'M')
+        assert nodes[4] in ('B', 'M')
+
+

=== modified file 'bzrlib/selftest/testlog.py'
--- bzrlib/selftest/testlog.py	2005-09-09 11:37:44 +0000
+++ bzrlib/selftest/testlog.py	2005-09-14 05:22:13 +0000
@@ -65,6 +65,25 @@
             self.assertEquals(expected, got)
 
     def test_cur_revno(self):
+        b = Branch('.', init=True)
+
+        lf = LogCatcher()
+        b.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
+    def test_cur_revno(self):
         b = Branch.initialize('.')
 
         lf = LogCatcher()

=== added file 'bzrlib/selftest/testmerge.py'
--- bzrlib/selftest/testmerge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testmerge.py	2005-09-05 07:10:59 +0000
@@ -0,0 +1,12 @@
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.merge import merge
+class TestMerge(TestCaseInTempDir):
+    """Test appending more than one revision"""
+    def test_pending(self):
+        br = Branch(".", init=True)
+        commit(br, "lala!")
+        self.assertEquals(len(br.pending_merges()), 0)
+        merge(['.', -1], [None, None])
+        self.assertEquals(len(br.pending_merges()), 0)

=== modified file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	2005-09-09 11:37:44 +0000
+++ bzrlib/selftest/testrevision.py	2005-09-14 05:22:13 +0000
@@ -128,12 +128,116 @@
         self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
                           'c@u-0-6', self.br2.revision_history())
 
+class TestIntermediateRevisions(TestCaseInTempDir):
+
+    def setUp(self):
+        from bzrlib.commit import commit
+        TestCaseInTempDir.setUp(self)
+        self.br1, self.br2 = make_branches()
+        commit(self.br2, "Commit eleven", rev_id="b@u-0-7")
+        commit(self.br2, "Commit twelve", rev_id="b@u-0-8")
+        commit(self.br2, "Commit thirtteen", rev_id="b@u-0-9")
+        self.br1.add_pending_merge(self.br2.revision_history()[6])
+        commit(self.br1, "Commit fourtten", rev_id="a@u-0-6")
+        self.br2.add_pending_merge(self.br1.revision_history()[6])
+        commit(self.br2, "Commit fifteen", rev_id="b@u-0-10")
+
+        from bzrlib.revision import MultipleRevisionSources
+        self.sources = MultipleRevisionSources(self.br1, self.br2)
+
+    def intervene(self, ancestor, revision, revision_history=None):
+        from bzrlib.revision import get_intervening_revisions
+        return get_intervening_revisions(ancestor,revision, self.sources, 
+                                         revision_history)
+
+    def test_intervene(self):
+        """Find intermediate revisions, without requiring history"""
+        from bzrlib.errors import NotAncestor, NoSuchRevision
+        assert len(self.intervene('a@u-0-0', 'a@u-0-0')) == 0
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
+                         ['a@u-0-1', 'a@u-0-2'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-3'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3'])
+        self.assertEqual(self.intervene('b@u-0-3', 'a@u-0-3'), 
+                         ['b@u-0-4', 'a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-2', 'a@u-0-3', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-5', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'a@u-0-5'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+                         self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'b@u-0-6'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+                          'b@u-0-5'])
+        self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
+                         self.br2.revision_history()), 
+                         ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10'), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10', 
+                                        self.br2.revision_history()), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertRaises(NotAncestor, self.intervene, 'b@u-0-10', 'b@u-0-6', 
+                          self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'c@u-0-10', 
+                          'b@u-0-6', self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
+                          'c@u-0-6', self.br2.revision_history())
+
 
 class TestCommonAncestor(TestCaseInTempDir):
     """Test checking whether a revision is an ancestor of another revision"""
 
+    def test_old_common_ancestor(self):
+        """Pick a resonable merge base using the old functionality"""
+        from bzrlib.revision import find_present_ancestors
+        from bzrlib.revision import old_common_ancestor as common_ancestor
+        from bzrlib.revision import MultipleRevisionSources
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = MultipleRevisionSources(br1, br2)
+
+        expected_ancestors_list = {revisions[3]:(0, 0), 
+                                   revisions[2]:(1, 1),
+                                   revisions_2[4]:(2, 1), 
+                                   revisions[1]:(3, 2),
+                                   revisions_2[3]:(4, 2),
+                                   revisions[0]:(5, 3) }
+        ancestors_list = find_present_ancestors(revisions[3], sources)
+        assert len(expected_ancestors_list) == len(ancestors_list)
+        for key, value in expected_ancestors_list.iteritems():
+            self.assertEqual(ancestors_list[key], value, 
+                              "key %r, %r != %r" % (key, ancestors_list[key],
+                                                    value))
+
+        self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
+                          revisions[0])
+        self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[1], revisions[1], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[2], revisions_2[4], sources),
+                          revisions[2])
+        self.assertEqual(common_ancestor(revisions[3], revisions_2[4], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
+                          revisions[4])
+        self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
+                          revisions_2[5])
+
     def test_common_ancestor(self):
-        from bzrlib.revision import find_present_ancestors, common_ancestor
+        """Pick a reasonable merge base"""
+        from bzrlib.revision import find_present_ancestors
+        from bzrlib.revision import common_ancestor
         from bzrlib.revision import MultipleRevisionSources
         br1, br2 = make_branches()
         revisions = br1.revision_history()
@@ -168,4 +272,5 @@
         self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
                           revisions[4])
         self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
-                          revisions_2[5])
+                          revisions[4])
+

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-09-09 11:37:44 +0000
+++ bzrlib/selftest/versioning.py	2005-09-14 05:22:13 +0000
@@ -17,7 +17,6 @@
 
 """Tests of simple versioning operations"""
 
-# TODO: test adding a file whose directory is not versioned
 # TODO: test trying to commit within a directory that is not yet added
 
 
@@ -55,11 +54,11 @@
         self.assertEquals(delta.added[0][0], 'foo')
         self.failIf(delta.modified)
 
-    def test_add_in_unversioned(self):
+    def test_branch_add_in_unversioned(self):
         """Try to add a file in an unversioned directory.
 
-        smart_add may eventually add the parent as necessary, but simple
-        branch add doesn't do that.
+        "bzr add" adds the parent as necessary, but simple branch add
+        doesn't do that.
         """
         from bzrlib.branch import Branch
         from bzrlib.errors import NotVersionedError
@@ -75,7 +74,53 @@
         
         self.check_and_upgrade()
 
-        
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" should add the parent(s) as necessary.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
+        # Multiple unversioned parents
+        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), ['veil'])
+        run_bzr(['add', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), [])
+
+        # Check whacky paths work
+        self.build_tree(['cicatriz/', 'cicatriz/esp'])
+        eq(list(b.unknowns()), ['cicatriz'])
+        run_bzr(['add', 'inertiatic/../cicatriz/esp'])
+        eq(list(b.unknowns()), [])
+
+    def test_add_in_versioned(self):
+        """Try to add a file in a versioned directory.
+
+        "bzr add" should do this happily.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', '--no-recurse', 'inertiatic'])
+        eq(list(b.unknowns()), ['inertiatic/esp'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
     def test_subdir_add(self):
         """Add in subdirectory should add only things from there down"""
         
@@ -131,7 +176,6 @@
         
 
         
-        
 class SubdirCommit(TestCaseInTempDir):
 
     def test_subdir_commit(self):

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2005-08-22 16:31:16 +0000
+++ bzrlib/shellcomplete.py	2005-09-09 17:12:26 +0000
@@ -17,15 +17,15 @@
 
     from inspect import getdoc
     import commands
-    context, cmdclass = commands.get_cmd_class(cmdname)
+    cmdobj = commands.get_cmd_object(cmdname)
 
-    doc = getdoc(cmdclass)
+    doc = getdoc(cmdobj)
     if doc == None:
         raise NotImplementedError("sorry, no detailed shellcomplete yet for %r" % cmdname)
 
-    shellcomplete_on_option(cmdclass.takes_options, outfile = None)
-    for aname in cmdclass.takes_args:
-	    outfile.write(aname + '\n')
+    shellcomplete_on_option(cmdobj.takes_options, outfile = None)
+    for aname in cmdobj.takes_args:
+        outfile.write(aname + '\n')
 
 
 def shellcomplete_on_option(options, outfile = None):

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-08-27 04:42:41 +0000
+++ bzrlib/status.py	2005-09-07 23:14:30 +0000
@@ -21,16 +21,22 @@
                 show_ids=False,
                 to_file=None,
                 show_pending=True):
-    """Display single-line status for non-ignored working files.
+    """Display status for non-ignored working files.
 
-    show_all
-        If true, show unmodified files too.
+    show_unchanged
+        If set, includes unchanged files.
 
     specific_files
         If set, only show the status of files in this list.
 
+    show_ids
+        If set, includes each file's id.
+
     to_file
         If set, write to this file (default stdout.)
+
+    show_pending
+        If set, write pending merges.
     """
     import sys
     from bzrlib.delta import compare_trees

=== modified file 'testsweet.py'
--- testsweet.py	2005-08-30 01:35:40 +0000
+++ testsweet.py	2005-09-06 11:09:03 +0000
@@ -56,7 +56,6 @@
     # XXX: Not used yet
 
 
-
 class EarlyStoppingTestResultAdapter(object):
     """An adapter for TestResult to stop at the first first failure or error"""
 
@@ -173,7 +172,7 @@
     # but only a little. Folk not using our testrunner will
     # have to delete their temp directories themselves.
     if result.wasSuccessful():
-        if TestCaseInTempDir.TEST_ROOT:
+        if TestCaseInTempDir.TEST_ROOT is not None:
             shutil.rmtree(TestCaseInTempDir.TEST_ROOT) 
     else:
         print "Failed tests working directories are in '%s'\n" % TestCaseInTempDir.TEST_ROOT

